@@ -28,6 +28,10 @@ class ExperimentContextDelegate(metaclass=abc.ABCMeta):
     def experiment_ctx_delegate_send_row(self, row):
         raise NotImplementedError()
 
+    @abc.abstractmethod
+    def experiment_ctx_delegate_send_log(self, log: str):
+        raise NotImplementedError()
+
     @abc.abstractmethod
     def experiment_ctx_delegate_get_t(self) -> float:
         raise NotImplementedError()
@@ -48,6 +52,9 @@ class ExperimentContext:
     def send_row(self, row: dict):
         self._delegate.experiment_ctx_delegate_send_row(row)
 
+    def log(self, log: str):
+        self._delegate.experiment_ctx_delegate_send_log(log)
+
     @property
     def t(self) -> float:
         return self._delegate.experiment_ctx_delegate_get_t()
@@ -118,6 +125,7 @@ class IExperimentUI(metaclass=abc.ABCMeta):
         raise NotImplementedError()
 
     experiments: List[Type[ExperimentProtocol]]
+    log_queue: queue.Queue
 
     @abc.abstractmethod
     def launch(self):
@@ -145,6 +153,7 @@ class ExperimentController(ExperimentContextDelegate, ExperimentUIDelegate):
     _ctx: ExperimentContext
     _running = False
     _file = None
+    _log_file = None
 
     _experiment_thread = None
 
@@ -200,9 +209,11 @@ class ExperimentController(ExperimentContextDelegate, ExperimentUIDelegate):
         os.makedirs(dir, exist_ok=True)
         label = self._ui.experiment_label or self._running_experiment.name
         filename = label + "-" + datetime.datetime.now().strftime("%Y%m%d-%H%M%S") + ".csv"
+        log_filename = label + "-" + datetime.datetime.now().strftime("%Y%m%d-%H%M%S") + ".log"
         self._filename = dir / filename
         logger.info(f"Output file: {self._filename}")
         self._file = open(self._filename, "w", newline="")
+        self._log_file = open(dir / log_filename, "w", newline="")
         self._csv_writer = csv.writer(self._file, quoting=csv.QUOTE_NONNUMERIC)
 
         comment_lines = self._get_comment_line(self._running_experiment, self._options)
@@ -241,6 +252,9 @@ class ExperimentController(ExperimentContextDelegate, ExperimentUIDelegate):
         if self._file is not None:
             self._file.close()
             self._file = None
+        if self._log_file is not None:
+            self._log_file.close()
+            self._log_file = None
         logger.debug(f"stopped experiment")
 
     def _get_t(self) -> float:
@@ -259,6 +273,19 @@ class ExperimentController(ExperimentContextDelegate, ExperimentUIDelegate):
         row_list = [row.get(col) for col in cols]
         self._csv_writer.writerow(row_list)
 
+    def experiment_ctx_delegate_send_log(self, message):
+        t = self._get_t()
+        time = datetime.datetime.now()
+        self._ui.log_queue.put({
+            "t": t,
+            "time": time,
+            "message": message,
+        })
+        self._log_file.write(f"{time} t={t}: {message}\n")
+
+
+        # TODO: write to file
+
     def experiment_ctx_delegate_get_t(self) -> float:
         return self._get_t()
 
